<?php
/**
 * Handler: for managing sections around the project
 * @todo Make ORM-object of this class
 */
namespace EwTools\Handler;

use EwTools\Storage\ObjectDB;
use EwTools\Support\Str;

/**
 * Sections
 *
 * @link    http://infomaker.se
 * @package EwTools\Handler
 * @since   EwTools\Handler\Sections 0.1
 */
class Sections {
    
    /**
     * Contains the default section name
     *
     * @var string
     */
    protected static $default_section = 'Nyheter';
    
    /**
     * Constant for option name
     *
     * @var string
     */
    protected static $options_name = 'everyware_registered_sections';
    
    /**
     * Instance of this class
     *
     * @var \EwTools\Handler\Sections
     */
    public static $instance = null;
    
    /**
     * The default structure of a section
     *
     * @var array
     */
    protected static $defaults = [ 'slug' => '', 'name' => '', 'includes' => '' ];
    
    /**
     * The prefix added to the class created from the section
     *
     * @var string
     */
    protected static $class_prefix = 'section-';
    
    /**
     * A list of sections with slug as key
     *
     * @var array
     */
    protected $sections = [];
    
    /**
     * A list of sections with name as key
     *
     * @var array
     */
    protected $sections_names = [];
    
    /**
     * Sections constructor.
     *
     * @since 0.1
     */
    protected function __construct() {
        $this->sections       = static::getOption( 'by_slug' );
        $this->sections_names = static::getOption( 'by_name' );
    }
    
    /**
     * Function to add section to collection
     *
     * @param string $slug
     * @param string $name
     * @param string $includes
     *
     * @since 0.1
     * @return mixed
     */
    public static function add( $slug, $name, $includes = '' ) {
        return static::setAttribute( $slug, [
            'name'       => $name,
            'slug'       => $slug,
            'class_name' => static::createClass( $slug ),
            'includes'   => $includes
        ] );
    }
    
    /**
     * Function to add multiple sections at ones
     *
     * @param array $sections
     *
     * @since 0.1
     * @return void
     */
    public static function addSections( $sections ) {
        $slug_formatted_sections = [];
        $name_formatted_sections = [];
        
        foreach ( $sections as $data ) {
            $data = array_replace( static::$defaults, $data );
            if( Str::notEmpty( $data[ 'name' ] ) ) {
                $name = $data[ 'name' ];
                $slug = Str::isEmpty( $data[ 'slug' ] ) ? Str::slug( $name ) : $data[ 'slug' ];
                
                $section = [
                    'name'       => $name,
                    'slug'       => $slug,
                    'class_name' => static::createClass( $slug ),
                    'includes'   => $data[ 'includes' ]
                ];
                
                $slug_formatted_sections[ $slug ] = $section;
                $name_formatted_sections[ $name ] = $section;
            }
        }
        
        static::getInstance()->sections       = $slug_formatted_sections;
        static::getInstance()->sections_names = $name_formatted_sections;
        
        static::updateTable();
        static::updateBoardSectionClasses();
    }
    
    /**
     * Function to count the number of sections in the collection
     *
     * @since 0.1
     * @return int
     */
    public static function count() {
        return count( static::getInstance()->sections );
    }
    
    /**
     * Function to check if a section exists in the collection
     *
     * @param string $slug
     *
     * @since 0.1
     * @return bool
     */
    public static function exist( $slug ) {
        return array_key_exists( $slug, static::getInstance()->sections );
    }
    
    /**
     * Function to get a specific section by slug
     *
     * @param string $slug
     *
     * @since 0.1
     * @return bool
     */
    public static function get( $slug ) {
        return array_key_exists( $slug, static::getInstance()->sections ) ? static::$instance->sections[ $slug ] : false;
    }
    
    /**
     * Function to retrieve all sections
     *
     * @since 0.1
     * @return mixed
     */
    public static function getAll() {
        
        return static::getInstance()->sections;
    }
    
    /**
     * Function to get all sections by name
     *
     * @since 0.1
     * @return mixed
     */
    public static function getAllByName() {
        
        return static::getInstance()->sections_names;
    }
    
    /**
     * Function to fetch the classes registered in Everyboard settings
     *
     * @uses \EwTools\Storage\ObjectDB::get()
     *
     * @since 0.1
     * @return array
     */
    public static function getBoardSectionClasses() {
        return ObjectDB::get( 'board_custom_css_classes', [] );
    }
    
    /**
     * Function to get a specific section by name
     *
     * @param $name
     *
     * @since 0.1
     * @return bool|mixed
     */
    public static function getByName( $name ) {
        
        $sections = static::getInstance()->sections_names;
        if( array_key_exists( $name, $sections ) ) {
            return $sections[ $name ];
        }
        
        $filtered_arr = array_filter( $sections, function ( $section ) use ( $name ) {
            return strcasecmp( $name, $section[ 'name' ] ) === 0 || static::isIncluded( $name, $section );
        }, ARRAY_FILTER_USE_BOTH );
        
        return count( $filtered_arr ) > 0 ? array_values( $filtered_arr )[ 0 ] : false;
    }
    
    /**
     * Function to get a specific section class by slug
     *
     * @param $slug
     *
     * @since 0.1
     * @return string
     */
    public static function getClassName( $slug ) {
        if( false !== ( $section = static::get( $slug ) ) ) {
            return $section[ 'class_name' ];
        } else if( count( $sections = static::getAll() ) > 0 ) {
            return array_shift( $sections )[ 'class_name' ];
        }
        
        return '';
    }
    
    /**
     * Function to get a specific section class by name
     *
     * @param $name
     *
     * @since 0.1
     * @return string
     */
    public static function getClassByName( $name ) {
        if( false !== ( $section = static::getByName( $name ) ) ) {
            return $section[ 'class_name' ];
        } else if( false !== ( $section = static::getByName( static::$default_section ) ) ) {
            return $section[ 'class_name' ];
        } else if( count( $sections = static::getAll() ) > 0 ) {
            return array_shift( $sections )[ 'class_name' ];
        }
        
        return '';
    }
    
    /**
     * Function to return the default values of a section value
     *
     * @since 0.1
     * @return array
     */
    public static function getDefaults() {
        return static::$defaults;
    }
    
    /**
     * Get instance of this class
     *
     * @since 0.1
     * @return object
     */
    public static function getInstance() {
        if( static::$instance === null ) {
            static::$instance = new static;
        }
        
        return static::$instance;
    }
    
    /**
     * Function to retrieve all sections as select-box data
     *
     * @since 0.1
     * @return array
     */
    public static function getSelectData() {
        
        return array_map( function ( $section ) {
            return [
                'value' => $section[ 'slug' ],
                'text'  => $section[ 'name' ]
            ];
        }, static::getAll() );
    }
    
    /**
     * Initialize the Sections Handler
     *
     * @since 0.1
     * @return object
     */
    public static function init() {
        return static::getInstance();
    }
    
    /**
     * Function to matching a name in the included list of a section
     *
     * @param $name
     * @param $section
     *
     * @since 0.1
     * @return bool
     */
    public static function isIncluded( $name, $section ) {
        $includes = array_get( $section, 'includes', '' );
        if( Str::notEmpty( $includes ) ) {
            
            $multi_includes = preg_split( '/\s*(,|\|)\s*/', $includes );
            
            if( ! is_array( $multi_includes ) ) {
                return strcasecmp( $name, $multi_includes ) === 0;
            }
            
            foreach ( $multi_includes as $include ) {
                if( strcasecmp( $name, $include ) === 0 ) {
                    return true;
                }
            }
        }
        
        return false;
    }
    
    /**
     * Function to delete a section from the collection
     *
     * @param string $slug
     *
     * @since 0.1
     * @return bool
     */
    public static function remove( $slug ) {
        if( array_key_exists( $slug, static::getInstance()->sections ) ) {
            static::removeAttribute( $slug );
            
            return true;
        }
        
        return false;
    }
    
    /**
     * Function to remove all sections
     *
     * @since 0.1
     * @return void
     */
    public static function removeAll() {
        
        foreach ( static::getAll() as $key => $section ) {
            static::removeAttribute( $section[ 'slug' ] );
        }
        
        static::updateTable();
    }
    
    /**
     * Function to update section classes in Everyboard settings
     *
     * @uses \EwTools\Storage\ObjectDB::update()
     *
     * @since 0.1
     * @return void
     */
    public static function updateBoardSectionClasses() {
        $board_classes    = static::getBoardSectionClasses();
        $existing_classes = array_map( function ( $board_class ) {
            return $board_class[ 'name' ];
        }, $board_classes );
        
        foreach ( array_values( static::getAll() ) as $section ) {
            if( ! in_array( $section[ 'class_name' ], $existing_classes ) ) {
                $board_classes[] = static::createBoardSectionClass( $section[ 'name' ], $section[ 'class_name' ] );
            }
        }
        
        ObjectDB::update( 'board_custom_css_classes', $board_classes );
    }
    
    // Protected functions
    // ======================================================
    
    /**
     * Function for creating a class setting to match Everyboard classes
     *
     * @param $name
     * @param $class
     *
     * @since 0.1
     * @return array
     */
    protected static function createBoardSectionClass( $name, $class ) {
        // switch Name and Class values because of a Everyboard Bug.
        return [
            'name'    => $class,
            'class'   => $name,
            'row'     => 'true',
            'column'  => 'true',
            'article' => 'false',
            'widget'  => 'true',
            'id'      => uniqid( 'class_', false )
        ];
    }
    
    /**
     * Function to create a class used for section-specific style in the theme
     *
     * @param $slug
     *
     * @since 0.1
     * @return string
     */
    protected static function createClass( $slug ) {
        return Str::slug( Str::prepend( static::$class_prefix, $slug ) );
    }
    
    /**
     * Function to retrieve the sections from the options table
     *
     * @param string $name_suffix
     *
     * @uses \EwTools\Storage\ObjectDB::get()
     *
     * @since 0.1
     * @return mixed|void
     */
    protected static function getOption( $name_suffix = '' ) {
        $option_name = ( $name_suffix !== '' ) ? static::$options_name . '_' . $name_suffix : static::$options_name;
        
        return ObjectDB::get( $option_name, [] );
    }
    
    /**
     * Function to remove section from collection
     *
     * @param string $key
     *
     * @since 0.1
     * @return bool
     */
    protected static function removeAttribute( $key ) {
        
        $section = static::$instance->sections[ $key ];
        
        if( array_key_exists( $section[ 'name' ], static::$instance->sections_names ) ) {
            
            unset( static::$instance->sections_names[ $section[ 'name' ] ] );
        }
        
        unset( static::$instance->sections[ $key ] );
        
        return true;
    }
    
    /**
     * Function to save the data in the options table
     *
     * @param array  $data
     * @param string $name_suffix
     *
     * @uses \EwTools\Storage\ObjectDB::update()
     *
     * @since 0.1
     * @return bool
     */
    protected static function saveOption( $data, $name_suffix = '' ) {
        $option_name = ( $name_suffix !== '' ) ? static::$options_name . '_' . $name_suffix : static::$options_name;
        
        return ObjectDB::update( $option_name, $data );
    }
    
    /**
     * Function to set the values and save the options
     *
     * @param string $key
     * @param array  $value
     *
     * @since 0.1
     * @return mixed
     */
    protected static function setAttribute( $key, array $value ) {
        
        static::$instance->sections[ $key ]                   = $value;
        static::$instance->sections_names[ $value[ 'name' ] ] = $value;
        
        static::updateTable();
        
        return static::$instance->sections[ $key ];
    }
    
    /**
     * Function to save current section list
     *
     * @since 0.1
     * @return void
     */
    protected static function updateTable() {
        static::saveOption( static::$instance->sections, 'by_slug' );
        static::saveOption( static::$instance->sections_names, 'by_name' );
    }
}